package com.aeologic.adhoc.qr_utils;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.net.Uri;
import android.provider.MediaStore;
import android.util.Base64;
import android.util.Log;
import android.widget.Toast;

import com.aeologic.adhoc.qr_utils.activity.QRScannerActivity;
import com.aeologic.adhoc.qr_utils.utils.Utility;
import com.karumi.dexter.Dexter;
import com.karumi.dexter.MultiplePermissionsReport;
import com.karumi.dexter.PermissionToken;
import com.karumi.dexter.listener.PermissionRequest;
import com.karumi.dexter.listener.multi.MultiplePermissionsListener;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.List;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.app.Activity.RESULT_OK;
import static com.aeologic.adhoc.qr_utils.activity.QRScannerActivity.QR_CONTENT;

/**
 * QrUtilsPlugin
 */
public class QrUtilsPlugin implements MethodCallHandler, PluginRegistry.ActivityResultListener {
    /**
     * Plugin registration.
     */
    private static final String TAG = QrUtilsPlugin.class.getSimpleName();
    private static final String METHOD_CHANNEL = "com.aeologic.adhoc.qr_utils";
    private Result result;
    private int requestID = 0;
    private static final int REQUEST_SCAN_QR = 101;
    private static final int REQUEST_GENERATE_QR = 102;


    private Activity activity;

    public QrUtilsPlugin(Activity activity) {
        this.activity = activity;
    }

    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), METHOD_CHANNEL);
        QrUtilsPlugin cameraPlugin = new QrUtilsPlugin(registrar.activity());
        registrar.addActivityResultListener(cameraPlugin);
        channel.setMethodCallHandler(cameraPlugin);
    }

    @Override
    public void onMethodCall(MethodCall call, final Result result) {
        this.result = result;
        if (call.method.equals("scanQR")) {
            requestID = REQUEST_SCAN_QR;
            checkPermission();
        } else if (call.method.equals("generateQR")) {
            requestID = REQUEST_GENERATE_QR;
            String content = call.argument("content");
            Log.v(TAG,"QR_CONTENT: "+content);
            generateQR(content);
        } else {
            result.notImplemented();
        }
    }

    private void checkPermission() {
        Dexter.withActivity(activity)
                .withPermissions(
                        Manifest.permission.CAMERA,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE
                ).withListener(new MultiplePermissionsListener() {
            @Override
            public void onPermissionsChecked(MultiplePermissionsReport report) {
                if (report.areAllPermissionsGranted()) {
                    if (requestID == REQUEST_SCAN_QR) {
                        scanQR();
                    }
                } else {
                    Toast.makeText(activity, activity.getString(R.string.grant_all_permission), Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onPermissionRationaleShouldBeShown(List<PermissionRequest> permissions, PermissionToken token) {
                token.continuePermissionRequest();
            }
        }).check();
    }

    private void generateQR(final String content) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    final Bitmap qrBmp = Utility.generateQRCode(content);
                    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                    qrBmp.compress(Bitmap.CompressFormat.PNG, 100, byteArrayOutputStream);
                    final byte[] byteArray = byteArrayOutputStream.toByteArray();
                    final String qrBase64 = Base64.encodeToString(byteArray, Base64.DEFAULT);
                    activity.runOnUiThread(new Runnable() {
                        public void run() {
                            Log.v(TAG, "QR_BASE_64: " + qrBase64);
                            result.success(byteArray);
                            //qrBmp
                            //
                        }
                    });
                } catch (Exception e) {
                    activity.runOnUiThread(new Runnable() {
                        public void run() {
                            Toast.makeText(activity, activity.getString(R.string.process_failed), Toast.LENGTH_SHORT).show();
                        }
                    });
                    e.printStackTrace();
                }
            }
        }).start();
    }

    private void scanQR() {
        Intent intent = new Intent(activity, QRScannerActivity.class);
        activity.startActivityForResult(intent, REQUEST_SCAN_QR);
    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        Log.v(TAG, "onActivityResult()");
        if (resultCode == RESULT_OK && requestCode == REQUEST_SCAN_QR) {
            String content = data != null ? data.getStringExtra(QR_CONTENT) : null;
            Log.v(TAG, "QR_CONTENT: " + content);
            if (data != null) {
                result.success(content);
            } else {
                Toast.makeText(activity, activity.getString(R.string.process_failed), Toast.LENGTH_SHORT).show();
            }
        }
        return false;
    }
}